@@ -63,14 +63,16 @@
  *
  * Similar to GConf, the default values in GSettings schemas can be
  * localized, but the localized values are stored in gettext catalogs
- * and looked up with the domain that is specified in the gettext-domain
- * attribute of the <tag>schemalist</tag> or <tag>schema</tag> elements
- * and the category that is specified in the l10n attribute of the
- * <tag>key</tag> element.
+ * and looked up with the domain that is specified in the
+ * <tag class="attribute">gettext-domain</tag> attribute of the
+ * <tag class="starttag">schemalist</tag> or <tag class="starttag">schema</tag>
+ * elements and the category that is specified in the l10n attribute of the
+ * <tag class="starttag">key</tag> element.
  *
  * GSettings uses schemas in a compact binary form that is created
- * by the gschema-compile utility. The input is a schema description in
- * an XML format that can be described by the following DTD:
+ * by the <link linkend="glib-compile-schemas">glib-compile-schemas</link>
+ * utility. The input is a schema description in an XML format that can be
+ * described by the following DTD:
  * |[<![CDATA[
  * <!ELEMENT schemalist (schema*) >
  * <!ATTLIST schemalist gettext-domain #IMPLIED >
@@ -115,6 +117,12 @@
  * ]]>
  * ]|
  *
+ * At runtime, schemas are identified by their id (as specified
+ * in the <tag class="attribute">id</tag> attribute of the
+ * <tag class="starttag">schema</tag> element). The
+ * convention for schema ids is to use a dotted name, similar in
+ * style to a DBus bus name, e.g. "org.gnome.font-rendering".
+ *
  * <refsect2>
  *  <title>Binding</title>
  *   <para>
@@ -386,6 +394,8 @@ g_settings_init (GSettings *settings)
 void
 g_settings_delay (GSettings *settings)
 {
+  g_return_if_fail (G_IS_SETTINGS (settings));
+
   if (settings->priv->delayed)
     return;
 
@@ -489,6 +499,8 @@ g_settings_set_property (GObject      *object,
 gboolean
 g_settings_get_has_unapplied (GSettings *settings)
 {
+  g_return_val_if_fail (G_IS_SETTINGS (settings), FALSE);
+
   return settings->priv->delayed &&
          g_delayed_settings_backend_get_has_unapplied (
            G_DELAYED_SETTINGS_BACKEND (settings->priv->backend));
@@ -549,6 +561,7 @@ g_settings_class_init (GSettingsClass *class)
   /**
    * GSettings::changed:
    * @settings: the object on which the signal was emitted
+   * @key: the name of the key that changed
    *
    * The "changed" signal is emitted when a key has potentially changed.
    * You should call one of the g_settings_get() calls to check the new
@@ -728,6 +741,8 @@ g_settings_get_value (GSettings   *settings,
   GVariant *sval;
   gchar *path;
 
+  g_return_val_if_fail (G_IS_SETTINGS (settings), NULL);
+
   sval = g_settings_schema_get_value (settings->priv->schema, key, &options);
 
   if G_UNLIKELY (sval == NULL)
@@ -834,6 +849,8 @@ g_settings_set_value (GSettings   *settings,
   GVariant *sval;
   gchar *path;
 
+  g_return_val_if_fail (G_IS_SETTINGS (settings), FALSE);
+
   sval = g_settings_schema_get_value (settings->priv->schema, key, NULL);
   correct_type = g_variant_is_of_type (value, g_variant_get_type (sval));
   g_variant_unref (sval);
@@ -937,6 +954,8 @@ g_settings_is_writable (GSettings   *settings,
   gboolean writable;
   gchar *path;
 
+  g_return_val_if_fail (G_IS_SETTINGS (settings), FALSE);
+
   path = g_strconcat (settings->priv->path, name, NULL);
   writable = g_settings_backend_get_writable (settings->priv->backend, path);
   g_free (path);
@@ -955,7 +974,7 @@ g_settings_is_writable (GSettings   *settings,
  * <replaceable>base-path</replaceable> is the base path of @settings.
  *
  * The schema for the child settings object must have been declared
- * in the schema of @settings using a <tag>child</tag> element.
+ * in the schema of @settings using a <tag class="starttag">child</tag> element.
  *
  * Since: 2.26
  */
@@ -968,6 +987,8 @@ g_settings_get_child (GSettings   *settings,
   gchar *child_name;
   GSettings *child;
 
+  g_return_val_if_fail (G_IS_SETTINGS (settings), NULL);
+
   child_name = g_strconcat (name, "/", NULL);
   child_schema = g_settings_schema_get_value (settings->priv->schema,
                                               child_name, NULL);
@@ -1157,7 +1178,7 @@ g_settings_binding_key_changed (GSettings   *settings,
                                 gpointer     user_data)
 {
   GSettingsBinding *binding = user_data;
-  GValue value = {  };
+  GValue value = { 0, };
   GVariant *variant;
 
   g_assert (settings == binding->settings);
@@ -1179,28 +1200,13 @@ g_settings_binding_key_changed (GSettings   *settings,
   binding->running = FALSE;
 }
 
-static void
-g_settings_binding_writable_changed (GSettings   *settings,
-                                     const gchar *key,
-                                     gpointer     user_data)
-{
-  GSettingsBinding *binding = user_data;
-  gboolean writable;
-
-  g_assert (settings == binding->settings);
-  g_assert (key == binding->key);
-
-  writable = g_settings_is_writable (settings, key);
-  g_object_set (binding->object, "sensitive", writable, NULL);
-}
-
 static void
 g_settings_binding_property_changed (GObject          *object,
                                      const GParamSpec *pspec,
                                      gpointer          user_data)
 {
   GSettingsBinding *binding = user_data;
-  GValue value = {  };
+  GValue value = { 0, };
   GVariant *variant;
 
   g_assert (object == binding->object);
@@ -1240,9 +1246,15 @@ g_settings_binding_property_changed (GObject          *object,
  * The binding uses the default GIO mapping functions to map
  * between the settings and property values. These functions
  * handle booleans, numeric types and string types in a
- * straightforward way. Use g_settings_bind_with_mapping()
- * if you need a custom mapping, or map between types that
- * are not supported by the default mapping functions.
+ * straightforward way. Use g_settings_bind_with_mapping() if
+ * you need a custom mapping, or map between types that are not
+ * supported by the default mapping functions.
+ *
+ * Unless the @flags include %G_SETTINGS_BIND_NO_SENSITIVITY, this
+ * function also establishes a binding between the writability of
+ * @key and the "sensitive" property of @object (if @object has
+ * a boolean property by that name). See g_settings_bind_writable()
+ * for more details about writable bindings.
  *
  * Note that the lifecycle of the binding is tied to the object,
  * and that you can have only one binding per object property.
@@ -1302,10 +1314,11 @@ g_settings_bind_with_mapping (GSettings               *settings,
 {
   GSettingsBinding *binding;
   GObjectClass *objectclass;
-  gboolean bind_sensitive;
   gchar *detailed_signal;
   GQuark binding_quark;
 
+  g_return_if_fail (G_IS_SETTINGS (settings));
+
   objectclass = G_OBJECT_GET_CLASS (object);
 
   binding = g_slice_new0 (GSettingsBinding);
@@ -1328,6 +1341,19 @@ g_settings_bind_with_mapping (GSettings               *settings,
       return;
     }
 
+  if ((flags & G_SETTINGS_BIND_GET) && (binding->property->flags & G_PARAM_WRITABLE) == 0)
+    {
+      g_critical ("g_settings_bind: property '%s' on class '%s' is not writable",
+                  property, G_OBJECT_TYPE_NAME (object));
+      return;
+    }
+  if ((flags & G_SETTINGS_BIND_SET) && (binding->property->flags & G_PARAM_READABLE) == 0)
+    {
+      g_critical ("g_settings_bind: property '%s' on class '%s' is not readable",
+                  property, G_OBJECT_TYPE_NAME (object));
+      return;
+    }
+
   {
     GVariant *value;
 
@@ -1364,10 +1390,12 @@ g_settings_bind_with_mapping (GSettings               *settings,
       GParamSpec *sensitive;
 
       sensitive = g_object_class_find_property (objectclass, "sensitive");
-      bind_sensitive = sensitive && sensitive->value_type == G_TYPE_BOOLEAN;
+
+      if (sensitive && sensitive->value_type == G_TYPE_BOOLEAN &&
+          (sensitive->flags & G_PARAM_WRITABLE))
+        g_settings_bind_writable (settings, binding->key,
+                                  object, "sensitive", FALSE);
     }
-  else
-    bind_sensitive = FALSE;
 
   if (flags & G_SETTINGS_BIND_SET)
     {
@@ -1399,23 +1427,124 @@ g_settings_bind_with_mapping (GSettings               *settings,
       g_settings_binding_key_changed (settings, binding->key, binding);
     }
 
-  if (bind_sensitive)
-    {
-      detailed_signal = g_strdup_printf ("writable-changed::%s", key);
-      binding->writable_handler_id =
-        g_signal_connect (settings, detailed_signal,
-                          G_CALLBACK (g_settings_binding_writable_changed),
-                          binding);
-      g_free (detailed_signal);
-
-      g_settings_binding_writable_changed (settings, binding->key, binding);
-    }
-
   binding_quark = g_settings_binding_quark (property);
   g_object_set_qdata_full (object, binding_quark,
                            binding, g_settings_binding_free);
 }
 
+typedef struct
+{
+  GSettings *settings;
+  gpointer object;
+  const gchar *key;
+  const gchar *property;
+  gboolean inverted;
+  gulong handler_id;
+} GSettingsWritableBinding;
+
+static void
+g_settings_writable_binding_free (gpointer data)
+{
+  GSettingsWritableBinding *binding = data;
+
+  g_signal_handler_disconnect (binding->settings, binding->handler_id);
+  g_object_unref (binding->settings);
+}
+
+static void
+g_settings_binding_writable_changed (GSettings   *settings,
+                                     const gchar *key,
+                                     gpointer     user_data)
+{
+  GSettingsWritableBinding *binding = user_data;
+  gboolean writable;
+
+  g_assert (settings == binding->settings);
+  g_assert (key == binding->key);
+
+  writable = g_settings_is_writable (settings, key);
+
+  if (binding->inverted)
+    writable = !writable;
+
+  g_object_set (binding->object, binding->property, writable, NULL);
+}
+
+/**
+ * g_settings_bind_writable:
+ * @settings: a #GSettings object
+ * @key: the key to bind
+ * @object: a #GObject
+ * @property: the name of a boolean property to bind
+ * @inverted: whether to 'invert' the value
+ *
+ * Create a binding between the writability of @key in the
+ * @settings object and the property @property of @object.
+ * The property must be boolean; "sensitive" or "visible"
+ * properties of widgets are the most likely candidates.
+ *
+ * Writable bindings are always uni-directional; changes of the
+ * writability of the setting will be propagated to the object
+ * property, not the other way.
+ *
+ * When the @inverted argument is %TRUE, the binding inverts the
+ * value as it passes from the setting to the object, i.e. @property
+ * will be set to %TRUE if the key is <emphasis>not</emphasis>
+ * writable.
+ *
+ * Note that the lifecycle of the binding is tied to the object,
+ * and that you can have only one binding per object property.
+ * If you bind the same property twice on the same object, the second
+ * binding overrides the first one.
+ *
+ * Since: 2.26
+ */
+void
+g_settings_bind_writable (GSettings   *settings,
+                          const gchar *key,
+                          gpointer     object,
+                          const gchar *property,
+                          gboolean     inverted)
+{
+  GSettingsWritableBinding *binding;
+  gchar *detailed_signal;
+  GParamSpec *pspec;
+
+  g_return_if_fail (G_IS_SETTINGS (settings));
+
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), property);
+  if (pspec == NULL)
+    {
+      g_critical ("g_settings_bind_writable: no property '%s' on class '%s'",
+                  property, G_OBJECT_TYPE_NAME (object));
+      return;
+    }
+  if ((pspec->flags & G_PARAM_WRITABLE) == 0)
+    {
+      g_critical ("g_settings_bind_writable: property '%s' on class '%s' is not writable",
+                  property, G_OBJECT_TYPE_NAME (object));
+      return;
+    }
+
+  binding = g_slice_new (GSettingsWritableBinding);
+  binding->settings = g_object_ref (settings);
+  binding->object = object;
+  binding->property = g_intern_string (property);
+  binding->inverted = inverted;
+
+  detailed_signal = g_strdup_printf ("writable-changed::%s", key);
+  binding->handler_id =
+    g_signal_connect (settings, detailed_signal,
+                      G_CALLBACK (g_settings_binding_writable_changed),
+                      binding);
+  g_free (detailed_signal);
+
+  g_object_set_qdata_full (object, g_settings_binding_quark (property),
+                           binding, g_settings_writable_binding_free);
+
+  g_settings_binding_writable_changed (settings, binding->key, binding);
+}
+
 /**
  * g_settings_unbind:
  * @object: the object
